//		Copyright (c) 1999 by
//		Advanced Visual Systems Inc.
//		All Rights Reserved
//	
//	This software comprises unpublished confidential information of
//	Advanced Visual Systems Inc. and may not be used, copied or made
//	available to anyone, except in accordance with the license
//	under which it is furnished.
//	
//      This file is under Perforce control
//      $Id: //depot/express/dbproj/dbil/sdi_driver.hxx#1 $
//

// file: sdi_driver.hxx
// author: Patrick Li, Advanced Visual Systems
// purpose: Provide low level driver for accessing a Database API
//
//   This is the base driver for any spatial engine. To create a new driver, simply
//   subclass from this, and fill in the appropriate methods. You then need to modify the
//   SDI_Connect::Connect method to support instancing the new driver.
//
#ifndef SDI_DRIVER_DEFINED
#define SDI_DRIVER_DEFINED

#include "sdi_error.hxx"
#include "sdi_shp_arr.hxx"
#include "sdi_transaction.hxx"
#include "avs/dbil.h"

// Define structure to hold version info
//
typedef struct {
   int major;
   int minor;
   int bug;
   int release;
   char desc[64];
} SDI_Version;

// Structure to hold cursor information
//
typedef struct SDI_Cursor {
	short driver_type;						// 0=unknown, 1=oracle, 2=odbc
   char *table;                        // Name of table for this query
   void *driver_info;                  // Abstract pointer to API specific info
   char *shape_name;                   // Name of shape for this query if any
   short shape_indicator;              // Indicator for shape
   int field_id;                       // Holds field id from initial query
   char *field_name;                   // Name of field for this query if any
   short field_indicator;              // Indicator for field
   int num_attrs;                      // Number of attributes to retrieve
   SDI_Attribute *attrs;               // Attribute definitions
   long **indicators;                  // Indicators for attributes
//	short *value_len;							// Lengths of data value fetched 
//	short *row_rc;								// Row level return codes
	short has_long;							// Long or Long Raw columns in the select stmt
	int use_floating_type_only;
	int floating_type;
	int fixed_type;
   int rows_per_fetch;						// rows per fetch from server buffer in select stmt
	DB_OUTPUT_STAT output_stat;			// Statistics to be displayed in DB Kit
   // Fetch null value parameters
   //
   int           int_null_value;
   unsigned int  uint_null_value;
   float         float_null_value;
   double        double_null_value;
   char          *string_null_value;
   short         short_null_value;
   unsigned char byte_null_value;
} SDI_CURSOR;

// Define SDI_Driver class, this is the subclass for all spatial drivers
//   To define a new interface, subclass off of this, and overload the appropriate
//   methods.
//
class SDI_Driver : public SDI_Error_Tool {
public:
   // Constructor and Destructor
   //
   SDI_Driver();
   virtual ~SDI_Driver();
   
   // Connect and Disconnect from server
   //   (note: These methods should be called from methods that overload them)
   //
   virtual SDI_Status Connect( char *server, char *instance, char *database, char *user, char *password );
   virtual SDI_Status Disconnect();

   // Stream processing
   //
   virtual SDI_Status OpenCursor( SDI_CURSOR &cursor );
   virtual SDI_Status CloseCursor( SDI_CURSOR &cursor );

   // Transaction Processing
   //
   virtual SDI_Status StartTransaction();
   virtual SDI_Status CommitTransaction();
   virtual SDI_Status RollbackTransaction();
   virtual SDI_Status CancelTransaction();

   // Query and retrieve data from spatial engine, 
   //  overload as supported by a particular API.
   //   (note: These methods should NOT be called from methods that overload them)
   //
	virtual SDI_Status PrepareStmt(char *sql_statement, int query_timeout_secs, int rows_per_fetch);
	virtual SDI_Status ExecuteStmt(int iteration, int row_off);
   virtual SDI_Status Define( SDI_CURSOR &cursor);

   virtual SDI_Status Query( SDI_CURSOR &cursor, char *sql_statement );

   virtual SDI_Status Query( SDI_CURSOR &cursor, int num_tables, char **tables, char *where, 
                             char *shape, char *field, int num_attrs, char **attributes, 
                             SDI_Shape *filter_shape, SDI_FILTER_METHODS filter_method );
   
   virtual SDI_Status Fetch( SDI_CURSOR &cursor, SDI_Shape *shape, 
                             SDI_Attribute *attrs, int offset = 0 );

   virtual SDI_Status Insert( char *table, SDI_Shape *shape, 
                              int num_attrs, SDI_Attribute *attrs );

   virtual SDI_Status Insert( char *table, SDI_Shape_Array *shape, int num_attrs, 
                              SDI_Attribute *attrs, int stopOnError, SDI_Status_Callback cb );

   virtual SDI_Status Update( char *table, char *where, char *shape_column, SDI_Shape *shape, 
                              int num_attrs, SDI_Attribute *attrs, SDI_Shape *filter_shape, 
                              SDI_FILTER_METHODS filter_method, SDI_Status_Callback cb );

   virtual SDI_Status Delete( char *table, char *where, char *shape, char *field, 
                              SDI_Shape *filter_shape, SDI_FILTER_METHODS filter_method, 
                              SDI_Status_Callback cb );
   
   // Misc SQL
   //
   virtual SDI_Status ProcessSQL( char *sql_statement );

   // Data Management Functions
   //
	virtual SDI_Status GetSysInfo( int option, char * table_name, char ** sql);

   virtual SDI_Status GetUserTables( int permissions, int *num_tables, char ***tables );
   virtual SDI_Status FreeUserTables( int num_tables, char **tables );

   virtual SDI_Status DescribeTable( char *table, int *num_columns, SDI_COLUMN_DEF **column_defs );
   virtual SDI_Status FreeTableDescriptions( int num_columns, SDI_COLUMN_DEF *column_defs );
   
   // Utilities
   //
   const SDI_Version *GetVersion() { return &m_version; }

	// Alarm signal handling routine
	//
	void SigHandler();


protected:
   SDI_Version m_version;

   // Create private copy constructor and assignment operator 
   // to invalidate copys and assignments
   //
   SDI_Driver( const SDI_Driver & ) {}
   SDI_Driver &operator=(const SDI_Driver &) { return *this; }
};
#endif
